package fr.cnrs.mri.util.database;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Observable;


public class DatabaseUtil extends Observable implements Runnable {
	
	private static DatabaseUtil databaseUtil;
	private boolean isStopped;
	private int progressMax;
	private int progress;
	private boolean stoppedWithError;
	private DatabaseLoaderConfig config;
	private String database;
	private String error;

	private DatabaseUtil() { }
	
	public static DatabaseUtil getInstance() {
		if (databaseUtil == null) {
			databaseUtil = new DatabaseUtil();
		}
		return databaseUtil;
	}
	
	public Connection connection(DBConfiguration dbConfig) throws SQLException {
		Connection connect = DriverManager.getConnection(dbConfig.databaseDriver() + ":" +  
				dbConfig.databaseSystem() + "://" +
				dbConfig.databaseHost() + ":" + 
				dbConfig.databasePort() + "/" +
				dbConfig.databaseName(), 
				dbConfig.databaseUser(),
				dbConfig.databasePassword());
		return connect;
	}
	
	public void executeSqlScript(String filePath, DBConfiguration dbConfig) {
		this.changed("running");
		this.stoppedWithError = false;
		int numberOfLines = this.countLinesIn(filePath);
		this.setProgressMax(numberOfLines);
		String thisLine, sqlQuery = "";
		BufferedReader in = null;
		try {
			Connection connection = connection(dbConfig);
			in = new BufferedReader(new FileReader(filePath));
			Statement stmt = connection.createStatement();
			int counter = 0;
			this.setProgress(counter);
			this.isStopped = false;
			while (!this.isStopped() && (thisLine = in.readLine()) != null)	{
				thisLine = thisLine.trim();
				this.setProgress(++counter);
				//Skip comments and empty lines
				if (thisLine.length() == 0) continue;
				if(thisLine.charAt(0) == '-' || thisLine.startsWith("/*")) continue;
				sqlQuery = sqlQuery + " " + thisLine;
				//If one command complete
				if(sqlQuery.charAt(sqlQuery.length() - 1) == ';') {
					sqlQuery = sqlQuery.replace(';' , ' '); //Remove the ; since jdbc complains
						stmt.execute(sqlQuery);
						sqlQuery = "";
				}
			}
		}	
		catch(IOException ex) {
			this.error = ex.getMessage();
			ex.printStackTrace();
			this.stoppedWithError = true;
		} catch (SQLException e) {
			this.error = e.getMessage();
			e.printStackTrace();
			this.stoppedWithError = true;
		}
		finally {
			sqlQuery = "";
			this.setProgress(this.getProgressMax());
			this.stop();
			try {
				in.close();
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
	}
	
	private int countLinesIn(String filePath) {
		int lines = 0;
		BufferedReader in = null;
		try {
			in = new BufferedReader(new FileReader(filePath));
			while (!this.isStopped() && (in.readLine()) != null) lines++;
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		} finally {
			try {
				in.close();
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
		return lines;
	}

	public int getProgressMax() {
		return this.progressMax;
	}
	
	private void setProgressMax(int max) {
		this.progressMax = max;
		this.changed("progressMax");
	}

	private void setProgress(int counter) {
		this.progress = counter;
		this.changed("progress");
	}
	
	public int getProgress() {
		return this.progress;
	}

	private void changed(String string) {
		this.setChanged();
		this.notifyObservers(string);
		this.clearChanged();
	}

	public boolean isStopped() {
		return this.isStopped;
	}

	public void stop() {
		this.isStopped = true;
		if (!stoppedWithError) this.changed("running"); else this.changed("error");
	}

	@Override
	public void run() {
		this.isStopped = false;
		this.executeSqlScript(this.database, this.config);
	}

	public void setConfig(DatabaseLoaderConfig config) {
		this.config = config;
	}

	public void setDatabase(String selectedDatabase) {
		this.database  = selectedDatabase;
	}

	public String getError() {
		return this.error;
	}
	
	public void setError(String error) {
		this.error = error;
		this.changed("error");
	}
}
